"""NDG OAuth Paste utilities for example code

"""
__author__ = "P J Kershaw"
__date__ = "19/10/12"
__copyright__ = "(C) 2009 Science and Technology Facilities Council"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id:$"
from os import path
from threading import Thread
import optparse

from OpenSSL import crypto, SSL
from paste.deploy import loadapp
from paste.script.util.logging_config import fileConfig
import paste.httpserver


THIS_DIR = path.dirname(__file__)
SHARED_CONFIG_DIR  = path.join(THIS_DIR, 'shared_config')
PKI_DIR = path.join(SHARED_CONFIG_DIR, 'pki') 
CACERT_DIR = path.join(PKI_DIR, 'ca')


def serve_app(config_filepath):     
    defCertFilePath = path.join(PKI_DIR, 'host.pem')
    defPriKeyFilePath = path.join(PKI_DIR, 'host.pem')
        
    parser = optparse.OptionParser()
    parser.add_option("-p",
                      "--port",
                      dest="port",
                      default=5000,
                      type='int',
                      help="port number to run under")

    parser.add_option("-s",
                      "--with-ssl",
                      dest="with_ssl",
                      default='True',
                      help="Run with SSL")

    parser.add_option("-c",
                      "--cert-file",
                      dest='certFilePath',
                      default=defCertFilePath,
                      help="SSL Certificate file")

    parser.add_option("-k",
                      "--private-key-file",
                      default=defPriKeyFilePath,
                      dest='priKeyFilePath',
                      help="SSL private key file")

    parser.add_option("-f",
                      "--conf",
                      dest="configFilePath",
                      default=config_filepath,
                      help="Configuration file path")
    
    parser.add_option("-a",
                      "--with-ssl-client-auth",
                      dest="ssl_client_authn",
                      action='store_true',
                      default=True,
                      help="Set client authentication with SSL (requires -s "
                           "option")
    
    opt = parser.parse_args()[0]
    config_filepath = path.abspath(opt.configFilePath)
    
    if opt.with_ssl.lower() == 'true':
        ssl_context = SSL.Context(SSL.SSLv23_METHOD)
    
        ssl_context.set_session_id('oauthserver')
        ssl_context.use_privatekey_file(opt.priKeyFilePath)
        ssl_context.use_certificate_file(opt.certFilePath)
        
        ssl_context.load_verify_locations(None, CACERT_DIR)
        ssl_context.set_verify_depth(9)
        
        # Load the application from the Paste ini file configuration        
        fileConfig(config_filepath, 
                   defaults={'here': path.dirname(config_filepath)})
        app = loadapp('config:%s' % config_filepath)
        
        if opt.ssl_client_authn:
            # Wrap the application in middleware to set the SSL client certificate 
            # obtained from the SSL handshake in environ                
            app = OpenSSLVerifyCallbackMiddleware(app)
            _callback = app.create_ssl_callback()
            
            # Wrap in middleware to simulate Apache environment
            app = ApacheSSLVariablesMiddleware(app)
        
            ssl_context.set_verify(SSL.VERIFY_PEER, _callback)
            
        server = PasteDeployAppServer(app=app, 
                                      port=opt.port,
                                      ssl_context=ssl_context) 
    else:
        server = PasteDeployAppServer(config_filepath=config_filepath, 
                                      port=opt.port) 
    server.start()


class PasteDeployAppServer(object):
    """Wrapper to paste.httpserver to enable background threading"""
    
    def __init__(self, app=None, config_filepath=None, port=7443, host='0.0.0.0',
                 ssl_context=None):
        """Load an application configuration from config_filepath ini file and 
        instantiate Paste server object
        """       
        self.__thread = None
        
        if config_filepath:
            if app:
                raise KeyError('Set either the "config_filepath" or "app" '
                               'keyword but not both')
            
            fileConfig(config_filepath, 
                       defaults={'here':path.dirname(config_filepath)})
            app = loadapp('config:%s' % config_filepath)
            
        elif app is None:
            raise KeyError('Either the "config_filepath" or "app" keyword must '
                           'be set')
                       
        self.__paste_server = paste.httpserver.serve(app, host=host, port=port, 
                                                    start_loop=False, 
                                                    ssl_context=ssl_context)
    
    @property
    def pasteServer(self):
        return self.__paste_server
    
    @property
    def thread(self):
        return self.__thread
    
    def start(self):
        """Start server"""
        self.pasteServer.serve_forever()
        
    def startThread(self):
        """Start server in a separate thread"""
        self.__thread = Thread(target=PasteDeployAppServer.start, args=(self,))
        self.thread.start()
        
    def terminateThread(self):
        self.pasteServer.server_close()

 
class OpenSSLVerifyCallbackMiddleware(object):
    """Set peer certificate retrieved from PyOpenSSL SSL context callback in
    environ dict SSL_CLIENT_CERT item
    
    FOR TESTING PURPOSES ONLY - IT IS NOT THREAD SAFE
    """
    def __init__(self, app):
        self._app = app
        self.ssl_client_cert = None
        self.ssl_client_cert_dn = None
        self.ignore_pat = None
        
    def create_ssl_callback(self):
        """Make a SSL Context callback function and return it to the caller"""
        def _callback(conn, x509, errnum, errdepth, ok):
            if errdepth == 0:
                subject = x509.get_subject()
                components = subject.get_components()
                if self.ignore_pat not in [i[-1] for i in components]:
                    self.ssl_client_cert = crypto.dump_certificate(
                                                    crypto.FILETYPE_PEM, x509)
                    self.ssl_client_cert_dn = '/'+ '/'.join(
                                        ['%s=%s' % i for i in components])
            return ok
        
        return _callback
        
    def __call__(self, environ, start_response):
        """Set the latest peer SSL client certificate from the SSL callback
        into environ SSL_CLIENT_CERT key"""
        if self.ssl_client_cert:
            environ['SSL_CLIENT_CERT'] = self.ssl_client_cert
            environ['SSL_CLIENT_S_DN'] = self.ssl_client_cert_dn
            self.ssl_client_cert = None

        return self._app(environ, start_response)
    

class ApacheSSLVariablesMiddleware(object):
    """Simulate Apache SSL environment setting relevant environ variables"""
    def __init__(self, app):
        self._app = app
                                 
    def __call__(self, environ, start_response):
        environ['HTTPS'] = '1'
        return self._app(environ, start_response)



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"
import logging
from ConfigParser import SafeConfigParser
log = logging.getLogger(__name__)


class ClientRegistration(object):
    """
    An entry in the client register.
    """
    def __init__(self, name, client_id, client_secret, client_type,
                 redirect_uris, authentication_data):
        self.name = name
        self.id = client_id
        self.secret = client_secret
        self.type = client_type
        if redirect_uris:
            self.redirect_uris = [r.strip() for r in redirect_uris.split(',')]
        else:
            self.redirect_uris = []
        self.authentication_data = authentication_data


class ClientRegister(object):
    """
    Client reqister read from a configuration file
    """
    register = {}
    def __init__(self, config_file=None):
	"""
	@type config_file: basestring
	@param config_file: configuration file to be read.  If the file is
	None or null, no read is attempted.
	"""
        if config_file:
            config = SafeConfigParser()
            config.read(config_file)
            client_keys = config.get('client_register', 'clients').strip()

            if client_keys:
                for client_key in [k.strip() for k in client_keys.split(',')]:
                    self._create_client(config, client_key, 'client')

    def _create_client(self, config, client_key, prefix):
        client_section_name = prefix + ':' + client_key
        client_id = config.get(client_section_name, 'id')
        client_secret = None

        if config.has_option(client_section_name, 'secret'):
            client_secret = config.get(client_section_name, 'secret')

        client_registration = ClientRegistration(
            config.get(client_section_name, 'name'),
            client_id,
            client_secret,
            config.get(client_section_name, 'type'),
            config.get(client_section_name, 'redirect_uris'),
            config.get(client_section_name, 'authentication_data'))

        self.register[client_id] = client_registration

    def is_registered_client(self, client_id):
        """Determines if a client ID is in the client register.
        """
        if client_id not in self.register:
            return ('Client of id "%s" is not registered.' % client_id)
        return None

    def is_valid_client(self, client_id, redirect_uri):
        """Determines if a client ID is in the client register and the
        redirect_uri is registered for that client.
        """
        # Check if client ID is registered.
        if client_id not in self.register:
            return ('Client of id "%s" is not registered.' % client_id)

        client = self.register[client_id]

        if redirect_uri is None:
            if len(client.redirect_uris) != 1:
                return ('No redirect URI is registered for the client or '
                        'specified in the request.')

        if redirect_uri is not None and redirect_uri not in client.redirect_uris:
            return 'Redirect URI is not registered.'

        return None




"""OAuth 2.0 WSGI server middleware implements support for basic bearer 
tokens and also X.509 certificates as access tokens

OAuth 2.0 Authorisation Server
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

import json
import logging
import httplib
import urllib

from ndg.oauth.server.lib.access_token.make_access_token import \
                                                    make_access_token
from ndg.oauth.server.lib.oauth.access_token import AccessTokenRequest
from ndg.oauth.server.lib.oauth.authorize import (AuthorizeRequest, 
                                                  AuthorizeResponse)
from ndg.oauth.server.lib.oauth.oauth_exception import OauthException
from ndg.oauth.server.lib.register.access_token import AccessTokenRegister
from ndg.oauth.server.lib.register.authorization_grant import \
                                                    AuthorizationGrantRegister

log = logging.getLogger(__name__)


class AuthorizationServer(object):
    """
    Provides the core OAuth 2.0 server functions.
    """
    AUTHZ_HDR_ENV_KEYNAME = 'HTTP_AUTHORIZATION'
    BEARER_TOK_ID = 'Bearer'
    MAC_TOK_ID = 'MAC'
    TOKEN_TYPES = (BEARER_TOK_ID, MAC_TOK_ID)
    
    def __init__(self, client_register, authorizer, client_authenticator,
                 resource_register, resource_authenticator,
                 access_token_generator, config):
        self.client_register = client_register
        self.authorizer = authorizer
        self.client_authenticator = client_authenticator
        self.resource_register = resource_register
        self.resource_authenticator = resource_authenticator
        self.access_token_generator = access_token_generator
        self.access_token_register = AccessTokenRegister(config)
        self.authorization_grant_register = AuthorizationGrantRegister(config)

    def authorize(self, request, client_authorized):
        """Handle an authorization request.

        It is assumed that the caller has checked whether the user is
        authenticated and that the user has authorised the client and scope.

        Request query parameters (from 
        http://tools.ietf.org/html/draft-ietf-oauth-v2-22):

        response_type
              REQUIRED.  Value MUST be set to "code".
        client_id
              REQUIRED.  The client identifier as described in Section 2.2.
        redirect_uri
              OPTIONAL, as described in Section 3.1.2.
        scope
              OPTIONAL.  The scope of the access request as described by
              Section 3.3.
        state
              RECOMMENDED.  An opaque value used by the client to maintain
              state between the request and callback.  The authorization
              server includes this value when redirecting the user-agent back
              to the client.  The parameter SHOULD be used for preventing
              cross-site request forgery as described in Section 10.12.

        Response:
              application/x-www-form-urlencoded format:
        code
              REQUIRED.  The authorization code generated by the
              authorization server.  The authorization code MUST expire
              shortly after it is issued to mitigate the risk of leaks.  A
              maximum authorization code lifetime of 10 minutes is
              RECOMMENDED.  The client MUST NOT use the authorization code
              more than once.  If an authorization code is used more than
              once, the authorization server MUST deny the request and SHOULD
              attempt to revoke all tokens previously issued based on that
              authorization code.  The authorization code is bound to the
              client identifier and redirection URI.
        state
              REQUIRED if the "state" parameter was present in the client
              authorization request.  The exact value received from the
              client.

        @type request: webob.Request
        @param request: HTTP request object

        @type client_authorized: bool
        @param client_authorized: True if resource owner has authorized client

        @rtype: tuple: (str, int, str)
        @return: tuple (
                     redirect_uri
                     HTTP status if error
                     error description
                 )
        """
        log.debug("Starting authorization request")

        # Parameters should only be taken from the query string.
        params = request.GET
        auth_request = AuthorizeRequest(params.get('response_type', None),
                                        params.get('client_id', None),
                                        params.get('redirect_uri', None),
                                        params.get('scope', None),
                                        params.get('state', None))

        try:
            self.check_request(request, params, post_only=False)

            # Check for required parameters.
            required_parameters = ['response_type', 'client_id']
            for param in required_parameters:
                if param not in params:
                    log.error("Missing request parameter %s from params: %s",
                              param, params)
                    raise OauthException('invalid_request', 
                                         "Missing request parameter: %s" % param)

            if not client_authorized:
                raise OauthException('access_denied', 
                                     'User has declined authorization')

            response_type = params.get('response_type', None)
            if response_type != 'code':
                raise OauthException('unsupported_response_type', 
                                     "Response type %s not supported" % 
                                     response_type)

            client_error = self.client_register.is_valid_client(
                                                    auth_request.client_id, 
                                                    auth_request.redirect_uri)
            if client_error:
                log.error("Invalid client: %s", client_error)
                return (None, httplib.BAD_REQUEST, client_error)

            # redirect_uri must be included in the request if the client has
            # more than one registered.
            client = self.client_register.register[auth_request.client_id]
            if len(client.redirect_uris) != 1 and not auth_request.redirect_uri:
                log.error("An authorization request has been made without a "
                          "return URI")
                return (None, 
                        httplib.BAD_REQUEST, 
                        ('An authorization request has been made without a '
                        'return URI.'))

            # Preconditions satisfied - generate grant.
            (grant, code) = self.authorizer.generate_authorization_grant(
                                                                auth_request, 
                                                                request)
            auth_response = AuthorizeResponse(code, auth_request.state)

            if not self.authorization_grant_register.add_grant(grant):
                log.error('Registering grant failed')
                raise OauthException('server_error', 
                                     'Authorization grant could not be created')
        except OauthException, exc:
            log.error("Redirecting back after error: %s - %s", 
                      exc.error, exc.error_description)
            
            return self._redirect_after_authorize(auth_request, None, exc.error,
                                                  exc.error_description)

        log.debug("Redirecting back after successful authorization.")
        return self._redirect_after_authorize(auth_request, auth_response)

    def _redirect_after_authorize(self, 
                                  auth_request, 
                                  auth_response=None, 
                                  error=None, 
                                  error_description=None):
        """Redirects to the redirect URI after the authorization process as
        completed.

        @type resp: ndg.oauth.server.lib.oauth.authorize.AuthorizeRequest
        @param resp: OAuth authorize request
        
        @type resp: ndg.oauth.server.lib.oauth.authorize.AuthorizeResponse
        @param resp: OAuth authorize response

        @type error: str
        @param error: OAuth error

        @type error_description: str
        @param error_description: error description
        """
        # Check for inconsistencies that should be reported directly to the user.
        if not auth_response and not error:
            error = 'server_error'
            error_description = 'Internal server error'

        # Get the redirect URI.
        client = self.client_register.register[auth_request.client_id]
        redirect_uri = (
            auth_request.redirect_uri if auth_request.redirect_uri else \
                client.redirect_uris[0]
        )
        if not redirect_uri:
            return (
                None, 
                httplib.BAD_REQUEST,
                'An authorization request has been made without a return URI.')

        # Redirect back to client with authorization code or error.
        if error:
            url_parameters = [('error', error), 
                              ('error_description', error_description)]
        else:
            url_parameters = [('code', auth_response.code)]
            
        full_redirect_uri = self._make_combined_url(redirect_uri, 
                                                    url_parameters, 
                                                    auth_request.state)
        log.debug("Redirecting to URI: %s", full_redirect_uri)
        return(full_redirect_uri, None, None)

    @staticmethod
    def _make_combined_url(base_url, parameters, state):
        """Constructs a URL from a base URL and parameters to be included in a
        query string.
        @type base_url: str
        @param base_url: base URL to which to add query parameters

        @type parameters: dict
        @param parameters: parameter names and values

        @type state: str
        @param state: OAuth state parameter value, which whould not be URL
        encoded

        @rtype: str
        @return: full URL
        """
        url = base_url.rstrip('?')
        url_parts = [url]
        sep_with_ampersand = ('?' in url)
        if parameters:
            query_string = urllib.urlencode(parameters)
            url_parts.extend([('&' if (sep_with_ampersand) else '?'), 
                              query_string])
            sep_with_ampersand = True

        if state:
            url_parts.extend([('&' if (sep_with_ampersand) else '?'), 
                              'state=',
                              state])

        return ''.join(url_parts)

    def access_token(self, request):
        """
        Handles a request for an access token.

        Request parameters in post data (from 
        http://tools.ietf.org/html/draft-ietf-oauth-v2-22):

        The client makes a request to the token endpoint by adding the
        following parameters using the "application/x-www-form-urlencoded"
        format in the HTTP request entity-body:

        grant_type
              REQUIRED.  Value MUST be set to "authorization_code".
        code
              REQUIRED.  The authorization code received from the
              authorization server.
        redirect_uri
              REQUIRED, if the "redirect_uri" parameter was included in the
              authorization request as described in Section 4.1.1, and their
              values MUST be identical.

        Response:
              application/json format:
        access_token
              access token
        token_type
              token type
        expires_in
              lifetime of token in seconds
        refresh_token

        @type request: webob.Request
        @param request: HTTP request object

        @rtype: tuple: (str, int, str)
        @return: tuple (
                     OAuth JSON response
                     HTTP status if error
                     error description
                 )
        """
        log.debug("Starting access token request")

        try:
            # Parameters should only be taken from the body, not the URL query 
            # string.
            params = request.POST
            self.check_request(request, params, post_only=True)

            # Check that the client is authenticated as a registered client.
            client_id = self.client_authenticator.authenticate(request)
            if client_id is None:
                log.warn('Client authentication not performed')
            else:
                log.debug("Client id: %s", client_id)

            # redirect_uri is only required if it was included in the 
            # authorization request.
            required_parameters = ['grant_type', 'code']
            for param in required_parameters:
                if param not in params:
                    log.error("Missing request parameter %s from inputs: %s",
                              param, params)
                    raise OauthException(
                                    'invalid_request', 
                                    "Missing request parameter: %s" % param)

        except OauthException, exc:
            return (self._error_access_token_response(exc.error, 
                                                      exc.error_description), 
                    None, None)

        token_request = AccessTokenRequest(params.get('grant_type', None),
                                           params.get('code', None),
                                           params.get('redirect_uri', None))

        try:
            response = make_access_token(
                token_request, client_id, self.access_token_register,
                self.access_token_generator, self.authorization_grant_register,
                request)

        except OauthException, exc:
            return (self._error_access_token_response(exc.error, 
                                                      exc.error_description), 
                    None, None)

        if response:
            return self._access_token_response(response), None, None
        else:
            return (None, httplib.INTERNAL_SERVER_ERROR, 
                    'Access token generation failed.')

    def _access_token_response(self, resp):
        """Constructs the JSON response to an access token request.
        @type resp: ndg.oauth.server.lib.oauth.access_token.AccessTokenResponse
        @param resp: OAuth access token response

        @rtype: str
        @return JSON formatted response
        """
        log.debug("Responding successfully with access token.")
        content_dict = resp.get_as_dict()
        content = json.dumps(content_dict)
        return content

    def _error_access_token_response(self, error, error_description):
        """Constructs an error JSON response to an access token request.
        @type error: str
        @param error: OAuth error

        @type error_description: str
        @param error_description: error description

        @rtype: str
        @return JSON formatted response
        """
        log.error("Responding with error: %s - %s", error, error_description)

        error_dict = {'error': error}

        if error_description:
            error_dict['error_description'] = error_description

        error_content = json.dumps(error_dict)
        return error_content

    def check_request(self, request, params, post_only=False):
        """
        Checks that the request is valid in the following respects:
        o Must be over HTTPS.
        o Optionally, must use the POST method.
        o Parameters must not be repeated.
        If the request is directly from the client, the user must be
        authenticated - it is assumed that the caller has checked this.

        Raises OauthException if any check fails.

        @type request: webob.Request
        @param request: HTTP request object

        @type params: dict
        @param params: request parameters

        @type post_only: bool
        @param post_only: True if the HTTP method must be POST, otherwise False

        """
        if request.scheme != 'https':
            raise OauthException('invalid_request', 
                                 'Transport layer security must be used for '
                                 'this request.')

        if post_only and request.method != 'POST':
            raise OauthException('invalid_request', 
                                 'HTTP POST method must be used for this '
                                 'request.')

        # Check for duplicate parameters.
        param_counts = {}
        for key in params.iterkeys():
            count = param_counts.get(key, 0)
            param_counts[key] = count + 1

        for key, count in param_counts.iteritems():
            if count > 1:
                raise OauthException('invalid_request', 
                                     'Parameter "%s" is repeated.' % key)

    def check_token(self, request, scope=None):
        """
        Simple service that could be used to validate bearer tokens. It would
        be called from a resource service that trusts this authorization
        service. This is not part of the OAuth specification.

        Only POST parameters are accepted in the request, to avoid logging
        and caching of access tokens.

        Request parameters

        access_token
              REQUIRED.  Bearer token
        scope
              OPTIONAL.  Scope 

        Response:
              application/json format:
        status
              HTTP status indicating the access control decision
        user_name
              user identifier corresponding to access token
        error
              error as described in
              http://tools.ietf.org/html/draft-ietf-oauth-v2-22#section-5.2

        @type request: webob.Request
        @param request: HTTP request object

        @type scope: str
        @param scope: required scope

        @rtype: tuple: (str, int, str)
        @return: tuple (
                     OAuth JSON response
                     HTTP status
                     error description
                 )
        """
        # only allow POST params to avoid logging and caching of access tokens
        params = request.POST

        # Check that the client is authenticated as a registered client.
        resource_id = self.resource_authenticator.authenticate(request)
        if resource_id is None:
            log.warn('Resource authentication not performed')
        else:
            log.debug("Resource id: %s", resource_id)

        # Retrieve access token
        if 'access_token' not in params:
            error = 'invalid_request'
        else:
            access_token = params['access_token']
            if scope:
                required_scope = scope
            else:
                required_scope = params.get('scope', None)
            token, error = self.access_token_register.get_token(access_token,
                                                                required_scope)
        # Formulate response
        status = {'invalid_request': httplib.BAD_REQUEST,
                  'invalid_token': httplib.FORBIDDEN,
                  None: httplib.OK}.get(error, httplib.BAD_REQUEST)

        content_dict = {'status': status}
        if error:
            content_dict['error'] = error
        else:
            # TODO only get additional data when resource is allowed to
            content_dict['user_name'] = token.grant.additional_data.get('user_identifier')

        content = json.dumps(content_dict)
        return (content, status, error)

    def get_registered_token(self, request, scope=None):
        """
        Checks that a token in the request is valid. It would
        be called from a resource service that trusts this authorization
        service. 

        Request parameters:
              set in Authorization header (OAuth spec., Section 7.1 Access
              Token Types
        token type: Bearer or MAC
        access token: access token to obtain access

        Response:
              application/json format:
        status
              HTTP status indicating the access control decision
        error
              error as described in
              http://tools.ietf.org/html/draft-ietf-oauth-v2-22#section-5.2

        @type request: webob.Request
        @param request: HTTP request object

        @type scope: str
        @param scope: required scope

        @rtype: tuple: (str, int, str)
        @return: tuple (
                     access token
                     HTTP status
                     error description
                 )
        """
        authorization_hdr = request.environ.get(
                                        self.__class__.AUTHZ_HDR_ENV_KEYNAME)
        if authorization_hdr is None:
            log.error('No Authorization header present for request to %r',
                      request.path_url)
            error = 'invalid_request'
            token = None            
        else:
            authorization_hdr_parts = authorization_hdr.split()
            if len(authorization_hdr_parts) < 2:
                log.error('Expecting at least two Authorization header '
                          'elements for request to %r; '
                          'header is: %r', request.path_url, authorization_hdr)
                error = 'invalid_request'            
                
            token_type, access_token = authorization_hdr_parts[:2]
            
            # Currently only supports bearer type tokens
            if token_type != self.__class__.BEARER_TOK_ID:
                log.error('Token type retrieved is %r, expecting "Bearer" '
                          'type for request to %r', token_type)
                error = 'invalid_request'
            else:   
                token, error = self.access_token_register.get_token(
                                                                access_token, 
                                                                scope)

        status = {'invalid_request': httplib.BAD_REQUEST,
                  'invalid_token': httplib.FORBIDDEN,
                  'insufficient_scope': httplib.FORBIDDEN,
                  None: httplib.OK}.get(error, httplib.BAD_REQUEST)

        return token, status, error

    def is_registered_client(self, request):
        """Determines whether the client ID in the request is registered.
        @type request: WebOb.request
        @param request: request
        @rtype: tuple (basestring, basestring) or (NoneType, NoneType)
        @return: (error, error description) or None if client ID is found and
        registered
        """
        client_id = request.params.get('client_id', None)
        if not client_id:
            return 'invalid_request', 'Missing request parameter: client_id'
        else:
            error_description = self.client_register.is_registered_client(
                                                                    client_id)
            if error_description:
                return 'unauthorized_client', error_description
            
        return None, None




"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"
import ndg.oauth.server.lib.register.scopeutil as scopeutil


class ClientAuthorization(object):
    """
    Represents authorizations granted by the resource owner to clients.
    """
    def __init__(self, user, client_id, scope, is_authorized):
        self.user = user
        self.client_id = client_id
        self.scope = scopeutil.scopeStringToList(scope)
        self.is_authorized = is_authorized

    def eq_authz_basis(self, other):
        """Determines whether a requested client authorization is equivalent to
        a granted one.
        @type other: ClientAuthorization
        @param other: requested authorization
        @rtype: bool
        @return: True if the user and client ID are the same and if there are no
        requested scopes that are not granted, otherwise False
        """
        return (self.user == other.user
                and self.client_id == other.client_id
                and scopeutil.isScopeGranted(self.scope, other.scope))

    def __repr__(self):
        return "user: %s  client_id: %s  scope: %s  granted: %s" % (
                                                            self.user, 
                                                            self.client_id, 
                                                            self.scope, 
                                                            self.is_authorized)

class ClientAuthorizationRegister(object):
    """
    Register of authorizations granted by the resource owner to clients.
    """
    def __init__(self):
        self.register = {}

    def add_client_authorization(self, client_authorization):
        user_authorizations = self.register.setdefault(
                                            client_authorization.user, {})
        client_authorizations = user_authorizations.setdefault(
                                            client_authorization.client_id, [])
        for auth in client_authorizations:
            if auth.eq_authz_basis(client_authorization):
                auth.is_authorized = client_authorization.is_authorized

        user_authorizations[client_authorization.client_id].append(
                                            client_authorization)


    def is_client_authorized_by_user(self, user, client_id, scope):
        client_authorization = ClientAuthorization(user, client_id, scope, True)
        user_authorizations = self.register.get(user)
        if user_authorizations:
            client_authorizations = user_authorizations.get(client_id, [])
            # Assume small number of authorization types per user/client 
            # (probably typically one).
            for auth in client_authorizations:
                if auth.eq_authz_basis(client_authorization):
                    return auth.is_authorized
        return None

    def __repr__(self):
        s = []
        for uv in self.register.itervalues():
            for cv in uv.itervalues():
                s.append(cv.__repr__())
        return ' '.join(s)



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

from abc import ABCMeta, abstractmethod

class AccessTokenInterface(object):
    """
    Interface for access token generators. Implementations will create a
    specific type of token.
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self, lifetime, token_type, **kw):
        """
        @type lifetime: int
        @param lifetime: lifetimes of generated tokens in seconds

        @type token_type: str
        @param token_type: token type name

        @type kw:dict
        @param kw: additional keywords
        """
        pass

    @abstractmethod
    def get_access_token(self, token_request, grant, request):
        """
        @type token_request: ndgoauthserver.lib.access_token.AccessTokenRequest
        @param token_request: access token request

        @type grant: ndgoauthserver.lib.register.authorization_grant.AuthorizationGrant
        @param grant: authorization grant

        @type request: webob.Request
        @param request: HTTP request object

        @rtype: ndgoauthserver.lib.register.access_token.AccessToken
        @return: access token
        """
        return None



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "W van Engen"
__date__ = "01/11/12"
__copyright__ = "(C) 2011 FOM / Nikhef"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "wvengen+oauth2@nikhef.nl"
__revision__ = "$Id$"

from base64 import b64decode

from ndg.oauth.server.lib.authenticate.authenticator_interface import AuthenticatorInterface
from ndg.oauth.server.lib.oauth.oauth_exception import OauthException


class PasswordAuthenticator(AuthenticatorInterface):
    """
    Authenticator implementation that checks for a client/resource id+secret
    combination, either in the HTTP Authorization header, or in the request
    parameters, according to the OAuth 2 RFC, section 2.3.1

    @todo implement protection against brute force attacks (MUST)
    """

    def __init__(self, typ, register):
        super(AuthenticatorInterfacei, self).__init__(typ)
        self._register = register

    def authenticate(self, request):
        """
        Checks for id/secret pair in Authorization header, or else
        POSTed request parameters.
        @type request: webob.Request
        @param request: HTTP request object

        @rtype: str
        @return: id of authenticated client/resource
        
        Raise OauthException if authentication fails.
        """
        cid = secret = None
        if 'Authorization' in request.headers and request.headers['Authorization'].startswith('Basic'):
            cid, secret = b64decode(request.headers['Authorization'][6:]).split(':',1)

        elif 'client_id' in request.POST and 'client_secret' in request.POST:
            cid = request.POST['client_id']
            secret = request.POST['client_secret']

        if not cid or not secret:
            raise OauthException('invalid_%s' % self.typ,
				 'No %s password authentication supplied' % self.typ)

        for authorization in self._register.register.itervalues():
            if authorization.id == cid and authorization.secret == secret:
                return authorization.id

        raise OauthException('invalid_%s' % self.typ,
			     '%s access denied: %s' % (cid, self.typ))




"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

from abc import ABCMeta, abstractmethod
class AuthorizerInterface(object):
    """
    Interface for generation of authorization grants.
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self, lifetime, **kw):
        """
        @type lifetime: int
        @param lifetime: lifetimes of generated tokens in seconds

        @type kw:dict
        @param kw: additional keywords
        """
        pass

    @abstractmethod
    def generate_authorization_grant(self, auth_request, request):
        """Generates an authorization grant.
        @type auth_request: ndgoauthserver.lib.oauth.authorize.AuthorizeRequest
        @param auth_request: authorization request

        @type request: webob.Request
        @param request: HTTP request object

        @rtype: tuple (
            ndgoauthserver.lib.register.authorization_grant.AuthorizationGrant
            str
        )
        @return: tuple (
            authorization grant
            authorization code
        )
        """
        return None



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "07/03/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

import httplib
import logging
import urllib
import urlparse

from repoze.who.api import get_api
from webob import Request

from ndg.oauth.server.lib.register.client import ClientRegister
from ndg.oauth.server.lib.register.client_authorization import (
                            ClientAuthorization, ClientAuthorizationRegister)
from ndg.oauth.server.lib.render.configuration import RenderingConfiguration
from ndg.oauth.server.lib.render.factory import callModuleObject
from ndg.oauth.server.lib.render.renderer_interface import RendererInterface

log = logging.getLogger(__name__)

class AuthenticationFormMiddleware(object):
    """Middleware to display a login form and handle the response.
    """
    CLIENT_AUTHORIZATIONS_SESSION_KEY = 'oauth2_client_authorizations'
    PARAM_PREFIX = 'authenticationForm.'
    LAYOUT_PREFIX = 'layout.'
    # Configuration options
    AUTHENTICATION_CANCELLED_OPTION = 'login_cancelled'
    AUTHENTICATION_FORM_OPTION = 'login_form'
    BASE_URL_PATH_OPTION = 'base_url_path'
    CLIENT_REGISTER_OPTION = 'client_register'
    COMBINED_AUTHORIZATION_OPTION = 'combined_authorization'
    RENDERER_CLASS_OPTION = 'renderer_class'
    RETURN_URL_PARAM_OPTION = 'return_url_param'
    SESSION_KEY_OPTION = 'session_key_name'
    method = {
        '/login_form': 'login_form',
        '/login': 'login'
    }
    # Configuration option defaults
    PROPERTY_DEFAULTS = {
        BASE_URL_PATH_OPTION: '/authentication',
        COMBINED_AUTHORIZATION_OPTION: 'True',
        RENDERER_CLASS_OPTION: 'ndg.oauth.server.lib.render.genshi_renderer.GenshiRenderer',
        RETURN_URL_PARAM_OPTION: 'returnurl',
        SESSION_KEY_OPTION: 'beaker.session.oauth2authorization'
    }
    LAYOUT_PARAMETERS = ['heading',
                         'title',
                         'message',
                         'leftLogo',
                         'leftAlt',
                         'leftImage',
                         'leftLink',
                         'rightAlt',
                         'rightImage',
                         'rightLink',
                         'footerText',
                         'helpIcon',
                         'client_id',
                         'client_name',
                         'scope']

    def __init__(self, app, app_conf, prefix=PARAM_PREFIX, **local_conf):
        """
        Sets up the server depending on the configuration.

        @type app: WSGI application
        @param app: wrapped application/middleware

        @type app_conf: dict
        @param app_conf: application configuration settings - ignored - this
        method includes this arg to fit Paste middleware / app function 
        signature

        @type prefix: str
        @param prefix: optional prefix for parameter names included in the 
        local_conf dict - enables these parameters to be filtered from others
        which don't apply to this middleware

        @type local_conf: dict
        @param local_conf: attribute settings to apply
        """
        self._app = app
        self._renderingConfiguration = RenderingConfiguration(
                                                    self.LAYOUT_PARAMETERS,
                                                    prefix + self.LAYOUT_PREFIX,
                                                    local_conf)
        self._set_configuration(prefix, local_conf)
        self.client_register = ClientRegister(self.client_register_file)
        self.renderer = callModuleObject(self.renderer_class,
                                         objectName=None, moduleFilePath=None, 
                                         objectType=RendererInterface,
                                         objectArgs=None, objectProperties=None)

    def __call__(self, environ, start_response):
        """
        @type environ: dict
        @param environ: WSGI environment

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        log.debug("AuthenticationFormMiddleware.__call__ ...")

        req = Request(environ)

        # Get session.
        session = environ.get(self.session_env_key)
        if session is None:
            raise Exception(
                'AuthenticationFormMiddleware.__call__: No beaker session key '
                '"%s" found in environ' % self.session_env_key)

        # Determine what operation the URL specifies.
        actionPath = None
        log.debug("Request path_info: %s", req.path_info)
        if req.path_info.startswith(self.base_path):
            actionPath = req.path_info[len(self.base_path):]
        methodName = self.__class__.method.get(actionPath, '')
        if methodName:
            log.debug("Method: %s" % methodName)
            action = getattr(self, methodName)
            return action(req, session, start_response)
        elif self._app is not None:
            log.debug("Delegating to lower filter/application.")
            return self._app(environ, start_response)
        else:
            response = "OAuth 2.0 Authentication Filter - Invalid URL"
            start_response(self._get_http_status_string(httplib.NOT_FOUND),
                           [('Content-type', 'text/plain'),
                            ('Content-length', str(len(response)))
                            ])
            return [response]

    def login_form(self, req, session, start_response):
        """Displays the login form.

        @type req: webob.Request
        @param req: HTTP request object

        @type session: Beaker SessionObject
        @param session: session data

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        submit_url = req.application_url + self.base_path + '/login'
        return_url = req.params.get(self.return_url_param)
        c = {'return_url': return_url,
             'return_url_param': self.return_url_param,
             'submit_url': submit_url,
             'baseURL': req.application_url}

        # Include authorization details on form if authentication and
        # authorization are to be combined.
        if self.combined_authorization:
            c.update(self._parse_return_url(return_url))

        response = self.renderer.render(self.authentication_form,
                            self._renderingConfiguration.merged_parameters(c))
        start_response(self._get_http_status_string(httplib.OK),
           [('Content-type', 'text/html'),
            ('Content-length', str(len(response)))
            ])
        return [response]

    def login(self, req, session, start_response):
        """Handles submission of the login form.

        @type req: webob.Request
        @param req: HTTP request object

        @type session: Beaker SessionObject
        @param session: session data

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        if ('submit' in req.params) and ('cancel' not in req.params):
            username = req.params.get('username')
            password = req.params.get('password')
            credentials = {'login': username, 'password': password}
            repoze_who_api = get_api(req.environ)
            (identity, headers) = repoze_who_api.login(credentials)
            if identity is not None:
                logged_in_username = identity['repoze.who.userid']
                log.debug("Logged in using username %s as %s", username,
                          logged_in_username)
    
                if self.combined_authorization:
                    self._set_client_authorization(req, session,
                                                   logged_in_username)
                return_url = req.params.get(self.return_url_param)
                return self._redirect(return_url, start_response, headers)
            else:
                # Login failed - redirect to login form.
                return_url = req.params.get(self.return_url_param)
                form_url = (req.application_url + self.base_path +
                    '/login_form' + '?' +
                    urllib.urlencode({self.return_url_param: return_url}))
                return self._redirect(form_url, start_response)
        else:
            # User cancelled authentication - confirm this.
            c = {'baseURL': req.application_url}
            response = self.renderer.render(self.authentication_cancelled,
                            self._renderingConfiguration.merged_parameters(c))
            start_response(self._get_http_status_string(httplib.OK),
               [('Content-type', 'text/html'),
                ('Content-length', str(len(response)))
                ])
            return [response]

    def _parse_return_url(self, return_url):
        """Gets client information from the return URL.
        @type return_url: basestring
        @param return_url: return URL
        @rtype: basestring
        @return: String describing client and scope if present
        """
        if not return_url:
            return None
        u = urlparse.urlparse(return_url)
        query_params = urlparse.parse_qs(u.query)
        client_id = query_params.get('client_id', None)
        if client_id:
            client_id = client_id[0]
        scope = query_params.get('scope', None)
        if scope:
            scope = scope[0]
        client = self.client_register.register.get(client_id)
        result = None
        if client:
            result = {'client_name': client.name,
                      'client_id': client.id,
                      'scope': scope}
        return result

    def _set_client_authorization(self, req, session, username):
        """Sets the client as authorized.

        @type req: webob.Request
        @param req: HTTP request object

        @type session: Beaker SessionObject
        @param session: session data
        """
        client_authorizations = session.setdefault(
                                self.CLIENT_AUTHORIZATIONS_SESSION_KEY,
                                ClientAuthorizationRegister())
        client_id = req.params.get('client_id')
        scope = req.params.get('scope')
        log.debug(
            "Adding client authorization for client_id: %s  scope: %s  user: %s",
            client_id, scope, username)
        client_authorizations.add_client_authorization(
                        ClientAuthorization(username, client_id, scope, True))
        session[self.CLIENT_AUTHORIZATIONS_SESSION_KEY] = client_authorizations
        log.debug("### client_auth: %s", client_authorizations.__repr__())
        session.save()
        # Make available immediately to chained middleware.
        session.persist()

    def _set_configuration(self, prefix, local_conf):
        """Sets the configuration values.

        @type prefix: str
        @param prefix: optional prefix for parameter names included in the
        local_conf dict - enables these parameters to be filtered from others
        which don't apply to this middleware

        @type local_conf: dict
        @param local_conf: attribute settings to apply
        """
        cls = self.__class__
        self.base_path = cls._get_config_option(prefix, local_conf,
                                                cls.BASE_URL_PATH_OPTION)
        self.client_register_file = cls._get_config_option(prefix, local_conf,
                                                    cls.CLIENT_REGISTER_OPTION)
        combined_authorization = cls._get_config_option(prefix, local_conf,
                                            cls.COMBINED_AUTHORIZATION_OPTION)
        self.combined_authorization = (combined_authorization.lower() == 'true')
        self.renderer_class = cls._get_config_option(prefix, local_conf,
                                                     cls.RENDERER_CLASS_OPTION)
        self.return_url_param = cls._get_config_option(prefix, local_conf,
                                                    cls.RETURN_URL_PARAM_OPTION)
        self.session_env_key = cls._get_config_option(prefix, local_conf,
                                                      cls.SESSION_KEY_OPTION)
        self.authentication_cancelled = cls._get_config_option(prefix,
                                                               local_conf,
                                            cls.AUTHENTICATION_CANCELLED_OPTION)
        self.authentication_form = cls._get_config_option(prefix, local_conf,
                                                cls.AUTHENTICATION_FORM_OPTION)

    @staticmethod
    def _get_http_status_string(status):
        return ("%d %s" % (status, httplib.responses[status]))

    def _redirect(self, url, start_response, headers=[]):
        log.debug("Redirecting to %s", url)
        hdrs = [('Location', url.encode('ascii', 'ignore'))]
        hdrs.extend(headers)
        start_response(self._get_http_status_string(httplib.FOUND), hdrs)
        return []

    @classmethod
    def _get_config_option(cls, prefix, local_conf, key):
        value = local_conf.get(prefix + key,
                               cls.PROPERTY_DEFAULTS.get(key,None))
        log.debug("AuthenticationFormMiddleware configuration %s=%s",
                  key, value)
        return value

    @classmethod
    def filter_app_factory(cls, app, app_conf, **local_conf):
        return cls(app, app_conf, **local_conf)



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

import uuid

from ndg.oauth.server.lib.access_token.access_token_interface import AccessTokenInterface
from ndg.oauth.server.lib.register.access_token import AccessToken

class BearerTokenGenerator(AccessTokenInterface):
    def __init__(self, lifetime, token_type, **kw):
        """
        @type lifetime: int
        @param lifetime: lifetimes of generated tokens in seconds

        @type token_type: str
        @param token_type: token type name

        @type kw:dict
        @param kw: additional keywords
        """
        self.lifetime = lifetime
        self.token_type = token_type

    def get_access_token(self, token_request, grant, request):
        """
        Gets an access token with an ID that is a random UUID used as a bearer
        token.
        @type token_request: ndg.oauth.server.lib.access_token.AccessTokenRequest
        @param token_request: access token request

        @type grant: ndg.oauth.server.lib.register.authorization_grant.AuthorizationGrant
        @param grant: authorization grant

        @type request: webob.Request
        @param request: HTTP request object

        @rtype: ndg.oauth.server.lib.register.access_token.AccessToken
        @return: access token or None if an error occurs
        """
        token_id = uuid.uuid4().hex
        return AccessToken(token_id, token_request, grant, self.token_type, self.lifetime)



#!/usr/bin/env python
"""NDG OAuth test harness for Short-Lived Credential Service bearer token 
example

"""
__author__ = "P J Kershaw"
__date__ = "20/11/08"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id:$"  
from os import path
                                     
from ndg.oauth.server.examples.utils import serve_app
                                      
INI_FILENAME = "slcs_server_app.ini"


if __name__ == '__main__':    
    config_filepath = path.join(path.dirname(path.abspath(__file__)), 
                                INI_FILENAME) 
    serve_app(INI_FILENAME)



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

from ndg.oauth.server.lib.authenticate.authenticator_interface import AuthenticatorInterface

class TestAuthenticator(AuthenticatorInterface):
    def authenticate(self, params):
        """Test implementation of user authenticator.
        @type params: dict
        @param params: request parameters

        @rtype: bool
        @return: True if user authenticated
        """
        if params.get('identifier', None) and params.get('password', '') == 'test':
            return True
        return False



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

import httplib
import logging

from webob import Request

from ndg.oauth.server.lib.register.client import ClientRegister
from ndg.oauth.server.lib.register.client_authorization import (
                            ClientAuthorization, ClientAuthorizationRegister)
from ndg.oauth.server.lib.render.configuration import RenderingConfiguration
from ndg.oauth.server.lib.render.factory import callModuleObject
from ndg.oauth.server.lib.render.renderer_interface import RendererInterface

log = logging.getLogger(__name__)


class Oauth2AuthorizationMiddleware(object):
    """Middleware to handle user/resource owner authorization of clients within
    a session.
    On each invocation, sets the current authorizations in the WSGI environ.
    At a specific URL, provides a simple form for a user to set an authorization
    decision.
    """
    CLIENT_AUTHORIZATIONS_SESSION_KEY = 'oauth2_client_authorizations'
    SESSION_CALL_CONTEXT_KEY = 'oauth2_client_authorizations_context'
    PARAM_PREFIX = 'oauth2authorization.'
    LAYOUT_PREFIX = 'layout.'
    # Configuration options
    BASE_URL_PATH_OPTION = 'base_url_path'
    CLIENT_AUTHORIZATION_FORM_OPTION = 'client_authorization_form'
    CLIENT_AUTHORIZATIONS_KEY_OPTION = 'client_authorizations_key'
    CLIENT_REGISTER_OPTION = 'client_register'
    RENDERER_CLASS_OPTION = 'renderer_class'
    SESSION_KEY_OPTION = 'session_key_name'
    USER_IDENTIFIER_KEY_OPTION = 'user_identifier_key'
    method = {
        '/authorize': 'authorize',
        '/client_auth': 'client_auth'
    }
    # Configuration option defaults
    PROPERTY_DEFAULTS = {
        BASE_URL_PATH_OPTION: 'client_authorization',
        RENDERER_CLASS_OPTION: \
            'ndg.oauth.server.lib.render.genshi_renderer.GenshiRenderer',
        SESSION_KEY_OPTION: 'beaker.session.oauth2authorization',
        CLIENT_AUTHORIZATIONS_KEY_OPTION: 'client_authorizations',
        USER_IDENTIFIER_KEY_OPTION: 'REMOTE_USER'
    }
    LAYOUT_PARAMETERS = ['heading',
                         'title',
                         'message',
                         'leftLogo',
                         'leftAlt',
                         'leftImage',
                         'leftLink',
                         'rightAlt',
                         'rightImage',
                         'rightLink',
                         'footerText',
                         'helpIcon']

    def __init__(self, app, app_conf, prefix=PARAM_PREFIX, **local_conf):
        """
        Sets up the server depending on the configuration.

        @type app: WSGI application
        @param app: wrapped application/middleware

        @type app_conf: dict
        @param app_conf: application configuration settings - ignored - this
        method includes this arg to fit Paste middleware / app function 
        signature

        @type prefix: str
        @param prefix: optional prefix for parameter names included in the 
        local_conf dict - enables these parameters to be filtered from others
        which don't apply to this middleware

        @type local_conf: dict
        @param local_conf: attribute settings to apply
        """
        self._app = app
        self._renderingConfiguration = RenderingConfiguration(
                                                    self.LAYOUT_PARAMETERS,
                                                    prefix + self.LAYOUT_PREFIX,
                                                    local_conf)
        self._set_configuration(prefix, local_conf)
        self.client_register = ClientRegister(self.client_register_file)
        self.renderer = callModuleObject(self.renderer_class,
                                         objectName=None, moduleFilePath=None, 
                                         objectType=RendererInterface,
                                         objectArgs=None, objectProperties=None)

    def __call__(self, environ, start_response):
        """
        @type environ: dict
        @param environ: WSGI environment

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        log.debug("Oauth2AuthorizationMiddleware.__call__ ...")

        req = Request(environ)

        # Get session.
        session = environ.get(self.session_env_key)
        if session is None:
            raise Exception(
                'Oauth2AuthorizationMiddleware.__call__: No beaker session key '
                '"%s" found in environ' % self.session_env_key)

        # Determine what operation the URL specifies.
        actionPath = None
        log.debug("Request path_info: %s", req.path_info)
        if req.path_info.startswith(self.base_path):
            actionPath = req.path_info[len(self.base_path):]
            
        methodName = self.__class__.method.get(actionPath, '')
        if methodName:
            log.debug("Method: %s" % methodName)
            action = getattr(self, methodName)
            return action(req, session, start_response)
        
        elif self._app is not None:
            log.debug("Delegating to lower filter/application.")
            self._set_client_authorizations_in_environ(session, environ)
            return self._app(environ, start_response)
        
        else:
            response = "OAuth 2.0 Authorization Filter - Invalid URL"
            start_response(self._get_http_status_string(httplib.NOT_FOUND),
                           [('Content-type', 'text/plain'),
                            ('Content-length', str(len(response)))
                            ])
            return [response]

    def _set_client_authorizations_in_environ(self, session, environ):
        """
        Sets the current authorizations currently granted by the user in
        environ,
        @type session: Beaker SessionObject
        @param session: session data

        @type environ: dict
        @param environ: WSGI environment
        """
        client_authorizations = session.get(self.CLIENT_AUTHORIZATIONS_SESSION_KEY)
        if client_authorizations:
            log.debug("_set_client_authorizations_in_environ %r",
                      client_authorizations)
            environ[self.client_authorizations_env_key] = client_authorizations
        else:
            log.debug("%s not found in session", 
                      self.CLIENT_AUTHORIZATIONS_SESSION_KEY)

    def authorize(self, req, session, start_response):
        """
        Checks whether the user has already authorized the client and if not
        displays the authorization form.
        @type req: webob.Request
        @param req: HTTP request object

        @type session: Beaker SessionObject
        @param session: session data

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        client_id = req.params.get('client_id')
        scope = req.params.get('scope')
        user = req.params.get('user')
        original_url = req.params.get('original_url')
        log.debug("Client authorization request for client_id: %s  scope: %s  "
                  "user: %s", client_id, scope, user)

        client_authorizations = session.get(
                                        self.CLIENT_AUTHORIZATIONS_SESSION_KEY)
        client_authorized = None
        if client_authorizations:
            client_authorized = \
                client_authorizations.is_client_authorized_by_user(user, 
                                                                   client_id, 
                                                                   scope)

        if client_authorized is None:
            # No existing decision - let user decide.
            session[self.SESSION_CALL_CONTEXT_KEY] = {
                'original_url': original_url,
                'client_id': client_id,
                'scope': scope,
                'user': user
                }
            session.save()
            log.debug("Client not authorized by user - returning authorization "
                      "form.")
            return self._client_auth_form(client_id, scope, req, start_response)
        else:
            log.debug("Client already %s authorization by user.", 
                      ("granted" if client_authorized else "denied"))
            log.debug("Redirecting to %s", original_url)
            return self._redirect(original_url, start_response)


    def _client_auth_form(self, client_id, scope, req, start_response):
        """
        Returns a form for the user to enter an authorization decision.

        @type client_id: str
        @param client_id: client identifier as set in the client register

        @type scope: str
        @param scope: authorization scope

        @type req: webob.Request
        @param req: HTTP request object

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        client = self.client_register.register.get(client_id)
        if client is None:
            # Client ID is not registered.
            log.error("OAuth client of ID %s is not registered with the server",
                      client_id)

            response = (
                "OAuth client of ID %s is not registered with the server" %
                client_id)
        else:
            submit_url = req.application_url + self.base_path + '/client_auth'
            c = {'client_name': client.name,
                 'client_id': client_id,
                 'scope': scope,
                 'submit_url': submit_url,
                 'baseURL': req.application_url}
            response = self.renderer.render(self.client_authorization_form,
                            self._renderingConfiguration.merged_parameters(c))
        start_response(self._get_http_status_string(httplib.OK),
           [('Content-type', 'text/html'),
            ('Content-length', str(len(response)))
            ])
        return [response]

    def client_auth(self, req, session, start_response):
        """
        @type req: webob.Request
        @param req: HTTP request object

        @type session: Beaker SessionObject
        @param session: session data

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        call_context = session.get(self.SESSION_CALL_CONTEXT_KEY)
        if not call_context:
            log.error("No session context.")
            response = 'Internal server error'
            status_str = self._get_http_status_string(
                                                httplib.INTERNAL_SERVER_ERROR)
            start_response(status_str,
                           [('Content-type', 'text/html'),
                            ('Content-length', str(len(response)))
                            ])
            return [response]

        if 'submit' in req.params and 'cancel' not in req.params:
            log.debug("User authorized client.")
            granted = True
        else:
            log.debug("User declined authorization for client.")
            granted = False

        # Add authorization to those for the user.
        client_authorizations = session.setdefault(
                                        self.CLIENT_AUTHORIZATIONS_SESSION_KEY, 
                                        ClientAuthorizationRegister())
        client_id = call_context['client_id']
        scope = call_context['scope']
        user = call_context['user']
        log.debug("Adding client authorization for client_id: %s  scope: %s  "
                  "user: %s", client_id, scope, user)
        
        client_authorization = ClientAuthorization(user, client_id, scope, 
                                                   granted)
        client_authorizations.add_client_authorization(client_authorization)
        
        session[self.CLIENT_AUTHORIZATIONS_SESSION_KEY] = client_authorizations
        log.debug("### client_auth: %r", client_authorizations)
        session.save()

        original_url = call_context['original_url']
        log.debug("Redirecting to %s", original_url)
        return self._redirect(original_url, start_response)

    def _set_configuration(self, prefix, local_conf):
        """Sets the configuration values.

        @type prefix: str
        @param prefix: optional prefix for parameter names included in the
        local_conf dict - enables these parameters to be filtered from others
        which don't apply to this middleware

        @type local_conf: dict
        @param local_conf: attribute settings to apply
        """
        cls = self.__class__
        self.base_path = cls._get_config_option(
                                            prefix, 
                                            local_conf, 
                                            cls.BASE_URL_PATH_OPTION)
        self.client_register_file = cls._get_config_option(
                                            prefix, 
                                            local_conf, 
                                            cls.CLIENT_REGISTER_OPTION)
        self.renderer_class = cls._get_config_option(
                                            prefix, 
                                            local_conf, 
                                            cls.RENDERER_CLASS_OPTION)
        self.session_env_key = cls._get_config_option(
                                            prefix, local_conf, 
                                            cls.SESSION_KEY_OPTION)
        self.client_authorization_form = cls._get_config_option(
                                            prefix, 
                                            local_conf, 
                                            cls.CLIENT_AUTHORIZATION_FORM_OPTION)
        self.client_authorizations_env_key = cls._get_config_option(prefix, 
                                            local_conf, 
                                            cls.CLIENT_AUTHORIZATIONS_KEY_OPTION)
        self.user_identifier_env_key = cls._get_config_option(
                                                prefix, 
                                                local_conf, 
                                                cls.USER_IDENTIFIER_KEY_OPTION)

    @staticmethod
    def _get_http_status_string(status):
        return ("%d %s" % (status, httplib.responses[status]))

    def _redirect(self, url, start_response):
        log.debug("Redirecting to %s", url)
        start_response(self._get_http_status_string(httplib.FOUND),
                       [('Location', url.encode('ascii', 'ignore'))])
        return []

    @classmethod
    def _get_config_option(cls, prefix, local_conf, key):
        value = local_conf.get(prefix + key, cls.PROPERTY_DEFAULTS.get(key, None))
        log.debug("Oauth2AuthorizationMiddleware configuration %s=%s", key, value)
        return value

    @classmethod
    def filter_app_factory(cls, app, app_conf, **local_conf):
        return cls(app, app_conf, **local_conf)



"""Plugin package for repoze.who including comparison of passwords against
stored MD5 hash and a variant of the SQLAuthenticatorPlugin supplied with
repoze.who that can be configured via the repoze.wno configuration file.
Substantially based on repoze.who.plugins.sql from version 2.0.
"""
__author__ = "R B Wilkinson"
__date__ = "11/05/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id$'

import logging
log = logging.getLogger(__name__)

try:
    from hashlib import md5
except ImportError: # Python < 2.5
    from md5 import new as md5

from zope.interface import implements
from repoze.who.interfaces import IAuthenticator
from repoze.who.plugins.sql import (default_password_compare,
                        SQLAuthenticatorPlugin as DefaultSQLAuthenticatorPlugin)

def md5_password_compare(cleartext_password, stored_password_hash):
    """Password comparison function suitable for use with repoze.who, for
    passwords stored as MD5 hashes.
    @type cleartext_password: basestring
    @param cleartext_password: cleartext password
    @type stored_password_hash: basestring
    @rtype: bool
    @return: True if the MD5 hash formed from the cleartext password matches
    the supplied stored hash, otherwise False
    """
    log.debug("Checking for match of password against hash '%s'",
              stored_password_hash)
    digest = md5(cleartext_password).hexdigest()

    if stored_password_hash == digest:
        return True

    return False

def make_psycopg_conn_factory(**kw):
    """Constructor of PostgreSQL connection factory using psycopg2.
    @type kw: dict
    @param kw: configuration options: should include the PostgreSQL connection
    string with key 'connection_string'
    """
    def conn_factory():
        import psycopg2
        return psycopg2.connect(kw['connection_string'])
    return conn_factory

class SQLAuthenticatorPlugin(DefaultSQLAuthenticatorPlugin):
    """Extension of repoze.who.plugins.sql.SQLAuthenticatorPlugin that allows
    configuration from a repoze.who configuration file:
    o allows additional keywords to be passed in so that a connection string
      can be set for the connection factory
    o allows a format for the query string that does not cause problems with
      ConfigParser parsing of the configuration file.
    """
    implements(IAuthenticator)

    def __init__(self, query, conn_factory, compare_fn, **kw):
        """
        Differs from repoze.who.plugins.sql:SQLAuthenticatorPlugin.__init__ only
        in the presence of the kw parameter. This is available to the connection
        factory.
        @type query: basestring
        @param query: SQL query to return the username and p
        @type conn_factory: function
        @param conn_factory: database connection factory
        @type compare_fn: function
        @param compare_fn: function to compare supplied and stored passwords
        @type kw: dict
        @param kw: additional configuration options that may be used by the
        connection factory constructor
        """
        # statement should be pyformat dbapi binding-style, e.g.
        # "select user_id, password from users where login=%(login)s"
        # Unfortunately, repose.who 2.0 uses ConfigParser, not SafeConfigParser,
        # making it impossible to include a pyformat query in the configuration
        # file (since there is no escaping mechanism). Allow '#(' to be used
        # instead of '%('.
        self.query = query.replace('#(', '%(')
        self.conn_factory = conn_factory
        self.compare_fn = compare_fn or default_password_compare
        self.conn = None

def make_authenticator_plugin(query=None, conn_factory=None,
                              compare_fn=None, **kw):
    """
    Differs from repoze.who.plugins.sql.make_authenticator_plugin only in that
    kw is passed to SQLAuthenticatorPlugin.
    @type query: basestring
    @param query: SQL query to return the username and p
    @type conn_factory: function
    @param conn_factory: database connection factory
    @type compare_fn: function
    @param compare_fn: function to compare supplied and stored passwords
    @type kw: dict
    @param kw: additional configuration options that are passed to the
    SQLAuthenticatorPlugin constructor
    """
    from repoze.who.utils import resolveDotted
    if query is None:
        raise ValueError('query must be specified')
    if conn_factory is None:
        raise ValueError('conn_factory must be specified')
    try:
        conn_factory = resolveDotted(conn_factory)(**kw)
    except Exception, why:
        raise ValueError('conn_factory could not be resolved: %s' % why)
    if compare_fn is not None:
        compare_fn = resolveDotted(compare_fn)
    return SQLAuthenticatorPlugin(query, conn_factory, compare_fn, **kw)



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

import logging
import uuid

from ndg.oauth.server.lib.authorize.authorizer_interface import AuthorizerInterface
from ndg.oauth.server.lib.register.authorization_grant import AuthorizationGrant

log = logging.getLogger(__name__)

class Authorizer(AuthorizerInterface):
    """
    Authorizer implementation that uses random UUIDs as authorization grant
    codes.
    """
    def __init__(self, lifetime):
        self.lifetime = lifetime

    def generate_authorization_grant(self, auth_request, request):
        """Generates an authorization grant.
        @type auth_request: ndg.oauth.server.lib.oauth.authorize.AuthorizeRequest
        @param auth_request: authorization request

        @type request: webob.Request
        @param request: HTTP request object

        @rtype: tuple (
            ndg.oauth.server.lib.register.authorization_grant.AuthorizationGrant
            str
        )
        @return: tuple (
            authorization grant
            authorization code
        )
        """
        code = uuid.uuid4().hex
        grant = AuthorizationGrant(code, auth_request, self.lifetime)
        return (grant, code)



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

from datetime import datetime

from ndg.oauth.server.lib.oauth.oauth_exception import OauthException
from ndg.oauth.server.lib.oauth.access_token import AccessTokenResponse

AUTHORIZATION_CODE_GRANT_TYPE = 'authorization_code'

def make_access_token(token_request, client_id, access_token_register, access_token_generator,
                      authorization_grant_register, request):
    """
    Makes an access token based on a token request and using a given access
    token generator.
    Returns
      access token response or None if an error occurs that is not one of those
          that can be reported in an error response, i.e., internal server error
    """
    # Check token_request valid and matches a registered authorization grant.
    if token_request.grant_type != AUTHORIZATION_CODE_GRANT_TYPE:
        raise OauthException('invalid_request', 'Invalid grant_type')

    try:
        grant = authorization_grant_register.get_value(token_request.code)
    except KeyError:
        raise OauthException('invalid_grant', 'Invalid authorization code')

    if (grant.redirect_uri is not None) and (token_request.redirect_uri != grant.redirect_uri):
        raise OauthException('invalid_grant', 'Invalid redirect URI')

    if grant.granted:
        # Invalidate the associated token.
        if grant.token:
            grant.token.valid = False
        raise OauthException('invalid_grant', 'Token already granted for authorization grant')

    # Check whether expired.
    if grant.expires <= datetime.utcnow():
        raise OauthException('invalid_grant', 'Authorization grant expired')

    # Check that the grant is issued to the requesting client.
    # This requires that the client has authenticated itself so that the
    # client identity is known.
    # client_id is None if client authentication is not configured - this
    # signals that authentication is disabled for testing.
    if client_id and (grant.client_id != client_id):
        raise OauthException('invalid_grant', 'Token granted for different client')

    token = access_token_generator.get_access_token(token_request, grant, request)
    if not token:
        return None

    grant.granted = True
    grant.token = token

    response = AccessTokenResponse(token.token_id, token.token_type,
                                   token.lifetime, refresh_token=None)

    if access_token_register.add_token(token):
        return response
    else:
        return None



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

class OauthException(Exception):
    """
    Exception representing OAuth errors
    """
    def __init__(self, error, error_description):
        self.error = error
        self.error_description = error_description

    def __str__(self):
        return("%s: %s" % (self.error, self.error_description))



__author__ = "R B Wilkinson"
__date__ = "29/02/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

_long_description = """\
This is an OAuth 2.0 server library and WSGI middleware filter.

It supports simple string-based bearer token and a custom extension to enable 
the use of X.509 certificates as tokens.  The latter has been added for a
specialised use case to enable a SLCS (Short-lived Credential Service) to issue 
delegated X.509-based credentials with OAuth.

Releases
========
0.5.0
-----
Integrated enhancements from Willem Engen including:

 * password-based client authentication, which is a commonly used client
   authentication method
 * resource authentication for the check_token endpoint, to avoid brute-force
   attacks on token check; also provides a starting point for audience-restricted
   tokens and resource-restricted attribute release
 * return user attribute from check_token endpoint, so that the resource knows
   what the user is; attribute name user_name according to CloudFoundry

Resource and client authentication use the same classes, and now are instantiated
with a string indicating their use (to give meaningful log messages). The
client_authenticator interface was removed, since all authenticators can derive
directly from authenticator_interface, since they're both used for clients and
resources; they were also renamed to make that clear (removing _client).

In client_register.ini and resource_register.ini (the latter is new) the field
secret is optional.

Client code is unchanged.

0.4.0
-----
 * Revised examples in ndg.oauth.client.examples.  bearer_tok uses bearer token
   to secure access to a simple html page on a resource server, slcs is an 
   example protecting a short-lived credential service aka. Online Certificate 
   Authority.  This requires the ContrailOnlineCAService package and should be 
   used in conjunction with the equivalent example in the ndg_oauth_client 
   example.
 * Added discrete WSGI resource server middleware 
   ndg.oauth.server.wsgi.resource_server.Oauth2ResourceServerMiddleware
 * Includes support for bearer access token passed in Authorization header to
   resource server.
 
Prerequisites
=============
This has been developed and tested for Python 2.6 and 2.7.

Installation
============
Installation can be performed using easy_install or pip.  

Configuration
=============
Examples are contained in the examples/ sub-folder:

bearer_tok/:
  This configures a simple test application that uses string based tokens.
slcs/:
  Bearer token example protecting a Short-Lived Credential Service or OnlineCA.
  ContrailOnlineCAService package is needed for this example.
  
The examples should be used in conjunction with the ndg_oauth_client package.
"""

setup(
    name =                      'ndg_oauth_server',
    version =                   '0.5.0',
    description =               'OAuth 2.0 server',
    long_description =          _long_description,
    author =                    'R. B. Wilkinson',
    maintainer =         	    'Philip Kershaw',
    maintainer_email =          'Philip.Kershaw@stfc.ac.uk',
    #url ='',
    license =                   'BSD - See LICENCE file for details',
    install_requires =[
        "PasteScript",
        "Beaker",
        "WebOb",
        "repoze.who",
        "Genshi",
    ],
    extras_require = {'slcs_support': 'MyProxyClient'},
    packages = find_packages(),
    package_data =      {
        'ndg.oauth.server.templates': [
            '*.html', 'public/js/*.js', 'public/layout/*.css',
            'public/layout/*.png', 'public/layout/icons/*.png'
        ],
        'ndg.oauth.server.examples': [
            'README'
        ],
        'ndg.oauth.server.examples.bearer_tok': [
            'README', 'passwd', '*.ini', 'templates/*.html', 
            'static/layout/*.css', 'static/layout/*.png'
        ],
        'ndg.oauth.server.examples.slcs': [
            'README', 'passwd', '*.ini', 'templates/*.html', 
            'static/layout/*.css', 'static/layout/*.png'
        ],
        'ndg.oauth.server.examples.shared_config': [
            'README', 'pki/*.pem', 'pki/ca/*.0'
        ]
    },
    zip_safe = False,
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Topic :: Security',
        'Topic :: Internet',
        'Topic :: Scientific/Engineering',
        'Topic :: System :: Distributed Computing',
        'Topic :: System :: Systems Administration :: Authentication/Directory',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

from beaker.cache import CacheManager
from beaker.util import parse_cache_config_options

class RegisterBase(object):
    """
    Base class for persistent registers. Entries are stored in a Beaker cache.
    """
    def __init__(self, name, config):
        cacheMgr = CacheManager(**parse_cache_config_options(config))
        self.cache = cacheMgr.get_cache(name)

    def set_value(self, key, value):
        self.cache.put(key, value)

    def get_value(self, key):
        return self.cache.get(key)

    def has_key(self, key):
        return self.cache.has_key(key)

    def parse_config(self, prefix, name, config):
        base = ("%s.%s." % (prefix, name))
        cache_opts = {
            'cache.expire': config.get(base + 'expire', None),
            'cache.type': config.get(base + 'type', 'file'),
            'cache.data_dir': config.get(base + 'data_dir', 
                                         '/tmp/ndgoauth/cache/' + name),
            'cache.lock_dir': config.get(base + 'lock_dir', None)
            }
        return cache_opts



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

from ndg.oauth.server.lib.authenticate.authenticator_interface import AuthenticatorInterface


class NoopAuthenticator(AuthenticatorInterface):
    """
    Authenticator implementation that returns None implying that the
    client/resource should not be authenticated.
    """
    def authenticate(self, request):
        """
        Authenticator that always returns None
        Returning None implies client/resource id is not to be checked against grant.

        @type request: webob.Request
        @param request: HTTP request object

        @rtype: str
        @return: None
        """
        return None

